from os import environ
import logging
from .utils import running_on_edge
import sys

# Config variables
validMessageFormats = ["string", "json"]



##############################
# LOGGING SETTINGS
# Get log level from env variable
logLevel = logging.INFO
if environ.get('logLevel') is not None:
    if environ.get('logLevel')=="ERROR":
        logLevel = logging.ERROR
    elif environ.get('logLevel')=="DEBUG":
        logLevel = logging.DEBUG
    elif environ.get('logLevel')=="INFO":
        logLevel = logging.INFO
    else:
        raise ValueError("Logging configuration " + str(environ.get('logLevel')) + " not supported!")


if not running_on_edge():
    # Set logging configuration for Cluster executing
    logging.basicConfig(
        level=logLevel,
        format="%(asctime)s [%(levelname)s] %(message)s",
        handlers=[
            logging.StreamHandler()
        ]
    )
else:
   # Set logging configuration for Edge executing
    logging.basicConfig(
        filename=environ.get('logs_location'), #"/shared/logs.log",
        filemode='a',
        level=logLevel,
        format="%(asctime)s [%(levelname)s] %(message)s",
    )
    
    # Redirect std out and err to file
    sys.stdout = open(environ.get('logs_location'), 'a')
    sys.stderr = sys.stdout



##############################


import os
from ..task_args import get_cmd_arg

# Checks if a service is running on Edge
def running_on_edge():
    if "running_on_edge" in os.environ:
        if os.environ.get('running_on_edge')=="True":
            return True
    else:
        return False

def get_group_id():
    return get_cmd_arg("spring.cloud.stream.bindings.input.group")



from ..utils import packages_are_installed

import logging
if not packages_are_installed(["sqlalchemy"]): 
    message = "Please install sqlalchemy (https://pypi.org/project/?) if you want to use scdf."
    logging.error(message)
    exit(1)
else:

    from sqlalchemy import create_engine
    from sqlalchemy.sql import text
    import datetime
    from .task_status import TaskStatus
    from ..task_args import get_task_id, get_db_url
    import os

    class SCDF(TaskStatus):
        """Helper class to help manage Task's status in the SCDF DB. """
        
        task_status_type = os.path.basename(__file__).split('.py')[0]

        def __init__(self):
            self.task_id = get_task_id()
            self.engine = create_engine(get_db_url())
            #self.connection = self.engine.connect()

        def running(self):
            """Set the TASK_EXECUTION's START_TIME """
            now = datetime.datetime.now()
            start_task_statement = text(
                "UPDATE TASK_EXECUTION SET START_TIME=:start_time, EXIT_CODE=null, LAST_UPDATED=:last_updated  "
                "WHERE TASK_EXECUTION_ID=:task_id")
            #self.connection.execute(start_task_statement, start_time=now, last_updated=now, task_id=self.task_id)
            self.engine.execute(start_task_statement, start_time=now, last_updated=now, task_id=self.task_id)

        def completed(self):
            # self.task_id = get_task_id()
            # self.engine = create_engine(get_db_url())
            # self.connection = self.engine.connect()

            """Set the TASK_EXECUTION's END_TIME, EXIST_CODE=0 and EXIST_MESSAGE/ERROR_MESSAGE must be null """
            now = datetime.datetime.now()
            complete_task_statement = text(
                "UPDATE TASK_EXECUTION SET END_TIME=:end_time, EXIT_CODE=0, EXIT_MESSAGE=null, ERROR_MESSAGE=null, "
                "LAST_UPDATED=:last_updated  WHERE TASK_EXECUTION_ID=:task_id")
            #self.connection.execute(complete_task_statement, end_time=now, last_updated=now, task_id=self.task_id)
            self.engine.execute(complete_task_statement,  end_time=now, last_updated=now, task_id=self.task_id)

        def failed(self, exit_code, exit_message, error_message=''):
            """Set the TASK_EXECUTION's END_TIME, EXIST_CODE is the error code and EXIST_MESSAGE/ERROR_MESSAGE describe
            the error """
            now = datetime.datetime.now()
            complete_task_statement = text(
                "UPDATE TASK_EXECUTION SET END_TIME=:end_time, EXIT_CODE=:exit_code, EXIT_MESSAGE=:exit_message, "
                "  ERROR_MESSAGE=:error_message, LAST_UPDATED=:last_updated  "
                "WHERE TASK_EXECUTION_ID=:task_id")
            # self.connection.execute(complete_task_statement, end_time=now, exit_code=exit_code,
            #                         exit_message=exit_message, error_message=error_message, last_updated=now,
            #                         task_id=self.task_id)

            exit_message = (exit_message[:100] + '..') if len(exit_message) > 100 else exit_message

            self.engine.execute(complete_task_statement, end_time=now, exit_code=exit_code,
                                    exit_message=exit_message, error_message=error_message, last_updated=now,
                                    task_id=self.task_id)



from .http_status_server import HttpHealthServer
from .task_args import get_kafka_binder_brokers, get_input_channel, get_output_channel, get_args, get_cmd_arg

from .task_status.task_status import TaskStatus

from .streaming.utils import running_on_edge

#import streaming




import threading
from http.server import HTTPServer, BaseHTTPRequestHandler


class HttpHealthServer(BaseHTTPRequestHandler):

    def do_GET(self):
        if (self.path == '/actuator/health') or (self.path == '/actuator/info'):
            self.send_ok_response()
        else:
            self.send_missing_response()

        self.send_header('Content-type', 'text/html')
        self.end_headers()

    def send_ok_response(self):
        self.send_response(200)

    def send_missing_response(self):
        self.send_response(404)

    @staticmethod
    def run_thread(port=8080):
        http_server = HTTPServer(('', port), HttpHealthServer)
        thread = threading.Thread(name='httpd_server', target=http_server.serve_forever)
        thread.setDaemon(True)
        thread.start()



from .task_status import TaskStatus
import os

class Argo(TaskStatus):
    """Helper class to help manage Task's status in the SCDF DB. """
    
    task_status_type = os.path.basename(__file__).split('.py')[0]

    def __init__(self):
        pass

    def running(self):
        print("Task is running.")

    def completed(self):
        print("Task completed.")

    def failed(self, exit_code, exit_message, error_message=''):
        # print("Error! Task failed with exit code: " + str(exit_code))
        # print("Exit message: ", exit_message)
        # if error_message!="":
        #     print("Error message: ", error_message)
        print("Task failed.")
        exit(1)



import os
from abc import ABC, abstractmethod
from ..task_args import get_cmd_arg

class TaskStatus(ABC):
    """Helper class to help manage Task's status in the SCDF DB. """
    def __new__(cls, *args, **kw):

        # Check if running on SCDF, Argo or locally        
        if get_cmd_arg('spring.datasource.url') is not None:
            task_status_type = "scdf"
        elif os.environ.get('EXECUTION_ENGINE') == "argo":
            task_status_type= "argo"
        else:
            task_status_type = "local"

        # Create a map of all subclasses based on storage type property (present on each subclass)
        subclass_map = {subclass.task_status_type: subclass for subclass in cls.__subclasses__()}

        # Select the proper subclass based on
        subclass = subclass_map[task_status_type]
        instance = super(TaskStatus, subclass).__new__(subclass)
        return instance

    @abstractmethod
    def running(self):
        pass
    
    @abstractmethod
    def completed(self):
        pass

    @abstractmethod
    def failed(self, exit_code, exit_message, error_message=''):
        pass